package java.security;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.HashMap;
import java.io.ObjectStreamField;
import java.util.Map;
import java.io.Serializable;

final class PermissionsHash extends PermissionCollection implements Serializable
{
    private transient Map<Permission, Permission> permsMap;
    private static final long serialVersionUID = -8491988220802933440L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    PermissionsHash() {
        this.permsMap = new HashMap(11);
    }
    
    @Override
    public void add(Permission permission) {
        synchronized (this) {
            this.permsMap.put(permission, permission);
        }
    }
    
    @Override
    public boolean implies(Permission permission) {
        synchronized (this) {
            if ((Permission)this.permsMap.get(permission) == null) {
                Iterator<Permission> iterator = this.permsMap.values().iterator();
                while (iterator.hasNext()) {
                    if (((Permission)iterator.next()).implies(permission)) {
                        return true;
                    }
                    continue;
                }
                return false;
            }
            return true;
        }
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return Collections.enumeration(this.permsMap.values());
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable<Object, Object> hashtable = new Hashtable(this.permsMap.size() * 2);
        synchronized (this) {
            hashtable.putAll(this.permsMap);
        }
        objectOutputStream.putFields().put("perms", hashtable);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Hashtable<Object, Object> hashtable = (Hashtable)objectInputStream.readFields().get("perms", null);
        (this.permsMap = new HashMap(hashtable.size() * 2)).putAll(hashtable);
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("perms", Hashtable.class) };
    }
}
